@@ -179,6 +179,7 @@ sub _get_license_check_file {
                 'license-problem-gfdl-invariants' =>
                   \&_check_gfdl_license_problem,
                 'rfc-whitelist-filename' =>\&_rfc_whitelist_filename,
+                'php-source-whitelist' => \&_php_source_whitelist,
             );
             my @splitline = split(/\s*\~\~\s*/, $_[1], 5);
             my $syntaxerror = 'Syntax error in '.$filename;
@@ -634,6 +635,13 @@ sub find_cruft {
             close($fd);
         }
 
+        # here we check old upstream specification
+        # debian/upstream should be a directory
+        if (   $name eq 'debian/upstream'
+            || $name eq 'debian/upstream-metadata.yaml') {
+            tag 'debian-upstream-obsolete-path', $name;
+        }
+
         unless ($warned->{$name}) {
             for my $rule (@file_checks) {
                 next if ($rule->[2] and not $info->native);
@@ -910,9 +918,10 @@ sub _tag_gfdl {
 # return True in case of license problem
 sub _check_gfdl_license_problem {
     my (
-        $name,$basename,$block,
-        $blocknumber,$cleanedblock,$matchedkeyword,
-        $licenseproblemhash,$licenseproblem,%matchedhash
+        $source_pkg, $name,$basename,
+        $block,$blocknumber,$cleanedblock,
+        $matchedkeyword,$licenseproblemhash,$licenseproblem,
+        %matchedhash
     )= @_;
     my $rawgfdlsections  = $matchedhash{rawgfdlsections}  || '';
     my $rawcontextbefore = $matchedhash{rawcontextbefore} || '';
@@ -1013,9 +1022,10 @@ sub _check_gfdl_license_problem {
 # whitelist good rfc
 sub _rfc_whitelist_filename {
     my (
-        $name,$basename,$block,
-        $blocknumber,$cleanedblock,$matchedkeyword,
-        $licenseproblemhash,$licenseproblem,%matchedhash
+        $source_pkg, $name, $basename,
+        $block,$blocknumber, $cleanedblock,
+        $matchedkeyword,$licenseproblemhash, $licenseproblem,
+        %matchedhash
     )= @_;
     my $lcname = lc($basename);
 
@@ -1029,6 +1039,22 @@ sub _rfc_whitelist_filename {
     return 1;
 }
 
+# whitelist php source
+sub _php_source_whitelist {
+    my (
+        $source_pkg, $name, $basename,
+        $block,$blocknumber, $cleanedblock,
+        $matchedkeyword,$licenseproblemhash, $licenseproblem,
+        %matchedhash
+    )= @_;
+
+    if($source_pkg =~ m,^php\d*(?:\.\d+)?$,xms) {
+        return 0;
+    }
+    tag $licenseproblem, $name;
+    return 1;
+}
+
 sub _clean_block {
     my ($text) = @_;
 
@@ -1041,6 +1067,13 @@ sub _clean_block {
     $text =~ s{ \\ \* \( [LR] \" }{\"}gxsm;
     $text =~ s{ \\ -}{-}gxsm;
 
+    # replace some shortcut (clisp)
+    $text =~ s{\(&fdl;\)}{ }gxsm;
+    $text =~ s{&fsf;}{free software foundation}gxsm;
+
+    # non breaking space
+    $text =~ s{&nbsp;}{ }gxsm;
+
     # replace some common comment-marker/markup with space
     $text =~ s{^\.\\\"}{ }gxms;               # man comments
 
@@ -1217,9 +1250,10 @@ sub _license_check {
 
         if(defined($licenseproblemdata->{'callsub'})) {
             my $subresult= $licenseproblemdata->{'callsub'}->(
-                $name, $basename, $block,
-                $blocknumber,$cleanedblock,$matchedkeyword,
-                $licenseproblemhash,$licenseproblem,%+
+                $source_pkg, $name, $basename,
+                $block,$blocknumber,$cleanedblock,
+                $matchedkeyword,$licenseproblemhash,$licenseproblem,
+                %+
             );
             if($subresult) {
                 $licenseproblemhash->{$licenseproblem} = 1;
