@@ -38,9 +38,24 @@ my $INCORRECT_LOCALE_CODES
 my $MULTIARCH_DIRS = Lintian::Data->new('common/multiarch-dirs', qr/\s++/,
     sub { return { 'dir' => $_[1], 'match' => qr/\Q$_[1]\E/ } });
 
-my $PRIVACY_BREAKER_WEBSITES
-  = Lintian::Data->new('files/privacy-breaker-websites',
-    qr/\s*\~\~/o,sub { return qr/$_[1]/xism });
+my $PRIVACY_BREAKER_WEBSITES= Lintian::Data->new(
+    'files/privacy-breaker-websites',
+    qr/\s*\~\~/o,
+    sub {
+        my ($regex, $tag, $suggest) = split(/\s*\~\~\s*/, $_[1], 3);
+        $tag = defined($tag) ? strip($tag) : '';
+        if (length($tag) == 0) {
+            $tag = $_[0];
+        }
+        my %ret = (
+            'tag' => $tag,
+            'regexp' => qr/$regex/xsm,
+        );
+        if (defined($suggest)) {
+            $ret{'suggest'} = $suggest;
+        }
+        return \%ret;
+    });
 
 my $PRIVACY_BREAKER_FRAGMENTS= Lintian::Data->new(
     'files/privacy-breaker-fragments',
@@ -597,7 +612,7 @@ sub run {
                         if ($pkg_config_arch eq $pkgconfig_dir) {
                             next MULTI_ARCH_DIR;
                         }
-                        if ($block =~ m{$regex}) {
+                        if ($block =~ m{\W$regex(\W|$)}xms) {
                             tag 'pkg-config-multi-arch-wrong-dir',$file,
                               'full text contains architecture specific dir',
                               $pkgconfig_dir;
@@ -651,9 +666,9 @@ sub run {
             # ---------------- .packlist files
             elsif ($fname =~ m,^usr/lib/perl.*/.packlist$,) {
                 tag 'package-installs-packlist', $file;
-            }elsif ($fname =~ m,^usr/lib/perl5/.*\.(?:pl|pm)$,) {
+            }elsif ($fname =~ m,^usr/lib/(?:[^/]+/)?perl5/.*\.(?:pl|pm)$,) {
                 push @nonbinary_perl_files_in_lib, $file;
-            }elsif ($fname =~ m,^usr/lib/perl5/.*\.(?:bs|so)$,) {
+            }elsif ($fname =~ m,^usr/lib/(?:[^/]+/)?perl5/.*\.(?:bs|so)$,) {
                 $has_binary_perl_file = 1;
             }
            # ---------------- /usr/lib -- needs to go after the other usr/lib/*
@@ -1397,23 +1412,28 @@ sub run {
                 my $finfo = $info->file_info($file) || '';
                 if ($finfo !~ m/gzip compressed/) {
                     tag 'gz-file-not-gzip', $file;
-                } elsif ($isma_same && $fname !~ m/\Q$arch\E/o) {
+                } else {
                     my $path = $info->unpacked($file);
                     my $buff;
+                    my $mtime;
                     open(my $fd, '<', $path);
                     # We need to read at least 8 bytes
                     if (sysread($fd, $buff, 1024) >= 8) {
                         # Extract the flags and the mtime.
                         #  NN NN  NN NN, NN NN NN NN  - bytes read
                         #  __ __  __ __,    $mtime    - variables
-                        my (undef, $mtime) = unpack('NN', $buff);
-                        if ($mtime){
-                            tag 'gzip-file-is-not-multi-arch-same-safe',$file;
-                        }
+                        (undef, $mtime) = unpack('NN', $buff);
                     } else {
                         fail "reading $file: $!";
                     }
                     close($fd);
+                    if ($mtime != 0) {
+                        if ($isma_same && $file !~ m/\Q$arch\E/o) {
+                            tag 'gzip-file-is-not-multi-arch-same-safe', $file;
+                        } else {
+                            tag 'package-contains-timestamped-gzip', $file;
+                        }
+                    }
                 }
             }
 
@@ -1764,7 +1784,7 @@ sub run {
             my $dirname = $dir->name;
             next if ($dirname =~ m{^var/} or $dirname =~ m{^etc/});
             if (scalar($dir->children) == 0) {
-                if (    $dirname ne 'usr/lib/perl5/'
+                if (    $dirname !~ m;^usr/lib/(?:[^/]+/)?perl5/$;
                     and $dirname ne 'usr/share/perl5/'
                     and $dirname !~ m;^usr/share/python-support/;) {
                     tag 'package-contains-empty-directory', $dirname;
@@ -1849,17 +1869,23 @@ sub _check_tag_url_privacy_breach {
             } elsif  ($relcontent eq 'bookmark') {
                 # see #746656
                 return;
+            } elsif ($relcontent eq 'generator-home') {
+                # generator-home is used by texinfo
+                return;
             }
         }
     }
 
     # track well known site
-    foreach my $breaker_tag ($PRIVACY_BREAKER_WEBSITES->all) {
-        my $regex= $PRIVACY_BREAKER_WEBSITES->value($breaker_tag);
-        if ($website =~ m{$regex}) {
-            unless (exists $privacybreachhash->{'tag-'.$breaker_tag}){
-                $privacybreachhash->{'tag-'.$breaker_tag}= 1;
-                tag $breaker_tag, $file, $url;
+    foreach my $breaker ($PRIVACY_BREAKER_WEBSITES->all) {
+        my $value = $PRIVACY_BREAKER_WEBSITES->value($breaker);
+        my $regex = $value->{'regexp'};
+        if ($website =~ m{$regex}mxs) {
+            unless (exists $privacybreachhash->{'tag-'.$breaker}) {
+                my $tag =  $value->{'tag'};
+                my $suggest = $value->{'suggest'} // '';
+                $privacybreachhash->{'tag-'.$breaker}= 1;
+                tag $tag, $file, $suggest, "($url)";
             }
             # do not go to generic case
             return;
@@ -1867,7 +1893,7 @@ sub _check_tag_url_privacy_breach {
     }
     # generic case
     unless (exists $privacybreachhash->{'tag-generic-'.$website}){
-        tag 'privacy-breach-generic', $file, $url;
+        tag 'privacy-breach-generic', $file, "($url)";
         $privacybreachhash->{'tag-generic-'.$website} = 1;
     }
     return;
