@@ -45,6 +45,18 @@ my %ROOT_INDEX_TEMPLATE = (
     'time'     => '22:55:34',
 );
 
+# A cache for (probably) the 5 most common permission strings seen in
+# the wild.
+# It may seem obscene, but it has an extreme "hit-ratio" and it is
+# cheaper vastly than perm2oct.
+my %PERM_CACHE = map { $_ => perm2oct($_) } (
+    '-rw-r--r--', # standard (non-executable) file
+    '-rwxr-xr-x', # standard executable file
+    'drwxr-xr-x', # standard dir perm
+    'drwxr-sr-x', # standard dir perm with suid (lintian-lab on lintian.d.o)
+    'lrwxrwxrwx', # symlinks
+);
+
 =head1 NAME
 
 Lintian::Collect::Package - Lintian base interface to binary and source package data collection
@@ -170,9 +182,9 @@ Needs-Info requirements for using I<file_info>: file-info
 
 sub file_info {
     my ($self, $file) = @_;
-    if (exists $self->{file_info}) {
-        return ${$self->{file_info}{$file}}
-          if exists $self->{file_info}->{$file};
+    if (my $cache = $self->{file_info}) {
+        return ${$cache->{$file}}
+          if exists $cache->{$file};
         return;
     }
     my %interned;
@@ -232,11 +244,9 @@ sub md5sums {
     while (my $line = <$fd>) {
         chop($line);
         next if $line =~ m/^\s*$/o;
-        $line =~ m/^(?'escaped'\\)?(?'md5sum'\S+)\s*(?'file'\S.*)$/o
+        $line =~ m/^(\\)?(\S+)\s*(\S.*)$/o
           or fail "syntax error in $md5f info file: $line";
-        my $zzsum = $+{'md5sum'};
-        my $zzescaped = defined($+{'escaped'});
-        my $zzfile = $+{'file'};
+        my ($zzescaped, $zzsum, $zzfile) = ($1, $2, $3);
         if($zzescaped) {
             $zzfile = dequote_name($zzfile);
         }
@@ -266,7 +276,7 @@ L</unpacked ([FILE])>.
 Note that the "root directory" (denoted by the empty string) will
 always be present, even if the underlying tarball omits it.
 
-Needs-Info requirements for using I<index>: index
+Needs-Info requirements for using I<index>: unpacked
 
 =cut
 
@@ -356,9 +366,9 @@ sub _fetch_extracted_dir {
 # sub _fetch_index_data Needs-Info none
 sub _fetch_index_data {
     my ($self, $field, $index, $indexown, $file) = @_;
-    if (exists $self->{$index}) {
-        return $self->{$field}->{$file}
-          if exists $self->{$index}->{$file};
+    if (my $cache = $self->{$field}) {
+        return $cache->{$file}
+          if exists $cache->{$file};
         return;
     }
     my $base_dir = $self->base_dir;
@@ -378,7 +388,16 @@ sub _fetch_index_data {
         my (%file, $perm, $owner, $name);
         ($perm,$owner,$file{size},$file{date},$file{time},$name)
           =split(' ', $_, 6);
-        $file{operm} = perm2oct($perm);
+        # This may appear to be obscene, but the call overhead of
+        # perm2oct is measurable on (e.g.) chromium-browser.  With
+        # the cache we go from ~1.5s to ~0.1s.
+        #   Of the 115363 paths here, only 306 had an "uncached"
+        # permission string (chromium-browser/32.0.1700.123-2).
+        if (exists($PERM_CACHE{$perm})) {
+            $file{operm} = $PERM_CACHE{$perm};
+        } else {
+            $file{operm} = perm2oct($perm);
+        }
         $file{type} = substr $perm, 0, 1;
 
         if ($num_idx) {
